from .loader import (
    DatasetLoader,
    load_heart_disease,
    load_titanic,
    load_boston_housing
)

__all__ = [
    "DatasetLoader",
    "load_heart_disease",
    "load_titanic",
    "load_boston_housing"
]
